# This file was generated by Rcpp::compileAttributes
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

colNorm_old2 <- function(M) {
    .Call('RcppGSLExample_colNorm_old2', PACKAGE = 'RcppGSLExample', M)
}

colNorm_old3 <- function(G) {
    .Call('RcppGSLExample_colNorm_old3', PACKAGE = 'RcppGSLExample', G)
}

colNorm <- function(G) {
    .Call('RcppGSLExample_colNorm', PACKAGE = 'RcppGSLExample', G)
}

